/*
 * ButtonsController.cpp
 *
 *  Created on: 14 nov. 2023
 *      Author: remi.heredero
 */

#include <board/ButtonsController.h>
#include "trace/trace.h"
#include "main.h"

ButtonsController::ButtonsController() {
	curentState_ = state::initial;
	cbProvider = nullptr;
	cbMethod = nullptr;
};

ButtonsController* ButtonsController::getInstance() {
	static ButtonsController buttonsController_;
	return &buttonsController_;
}

void ButtonsController::onIrq() {
	GEN(XFEvent(XFEvent::XFEventType::Event, event::evButtonIrq, this));
}

bool ButtonsController::registerCallback(
		interface::ButtonsControllerCallbackProvider *callbackProvider,
		interface::ButtonsControllerCallbackProvider::CallbackMethod callbackMethod) {

	if(callbackProvider == nullptr) return false;
	if(callbackMethod == nullptr) return false;

	cbProvider = callbackProvider;
	cbMethod = callbackMethod;

	return true;
}

XFEventStatus ButtonsController::processEvent() {
	eEventStatus eventStatus = XFEventStatus::Unknown;

	const XFEvent* ev = getCurrentEvent();
	XFEvent::XFEventType evType = ev->getEventType();
	int evid = ev->getId();

	oldState_ = curentState_;
	changeState = false;

	switch (curentState_) {
		case state::initial:
			if(evType == XFEvent::Initial) {
				curentState_ = state::checkButtons;
				changeState = true;
				eventStatus = XFEventStatus::Consumed;
			}
			break;

		case state::checkButtons:
			if (evid == event::evButtonIrq) {
				curentState_ = state::debounce;
				changeState = true;
				eventStatus = XFEventStatus::Consumed;
			}
			break;

		case state::debounce:
			if (evid == event::evTimeout) {
				curentState_ = state::checkButtons;
				changeState = true;
				eventStatus = XFEventStatus::Consumed;
			}
			break;
	}

	if(changeState) {
		switch (oldState_) { // onExit
			case state::initial:
				break;

			case state::checkButtons:
					break;

			case state::debounce:
				break;

		}

		switch (curentState_) { // onEntry
			case state::initial:
				break;

			case state::checkButtons:

				if(cbProvider != nullptr) {

					newState[0] = HAL_GPIO_ReadPin(BUTTON0_GPIO_Port, BUTTON0_Pin);
					newState[1] = HAL_GPIO_ReadPin(BUTTON1_GPIO_Port, BUTTON1_Pin);
					newState[2] = HAL_GPIO_ReadPin(BUTTON2_GPIO_Port, BUTTON2_Pin);
					newState[3] = HAL_GPIO_ReadPin(BUTTON3_GPIO_Port, BUTTON3_Pin);

					if(newState[0] != buttonState[0]) {
						buttonState[0] = newState[0];
						(cbProvider->*cbMethod)(0, !buttonState[0]);

					}

					if(newState[1] != buttonState[1]) {
						buttonState[1] = newState[1];
						(cbProvider->*cbMethod)(1, !buttonState[1]);
					}

					if(newState[2] != buttonState[2]) {
						buttonState[2] = newState[2];
						(cbProvider->*cbMethod)(2, !buttonState[2]);
					}

					if(newState[3] != buttonState[3]) {
						buttonState[3] = newState[3];
						(cbProvider->*cbMethod)(3, !buttonState[3]);
					}

				}
				break;

			case state::debounce:
				scheduleTimeout(event::evTimeout, 100);
				break;

		}
	}
	return eventStatus;
}
